//
//  ARGameViewController.swift
//
//  Created by Laura Sempere on 2/12/2017.
//  Copyright © 2017 laurasempere. All rights reserved.
//

import UIKit
import ARKit
import SceneKit

enum BodyType : Int {
    case bullet = 1
    case word = 2
}
let MAX_WORDS_PER_ROUND = 3

class ARGameViewController: UIViewController, Alertable {
    
    @IBOutlet weak var learntWordsCounterLabel: UILabel!
    @IBOutlet weak var sceneView: ARSCNView!
    let googleStorage = GoogleStorage()
    
    var learntWordsCounterText:String? {
       
        if let words = GameManager.instance.words {
            return "\(GameManager.instance.learntWords.count) \\ \(words.count)"
        } else {
            return nil
        }
    }
    var currentRound = 1
    
    override func viewDidLoad() {
        super.viewDidLoad()
        sceneView.delegate = self
        sceneView.showsStatistics = true
        let scene = SCNScene()
        sceneView.scene = scene
        self.sceneView.scene.physicsWorld.contactDelegate = self
        sceneView.antialiasingMode = .multisampling4X
        sceneView.autoenablesDefaultLighting = true
        updateLearnWordsCounterLabel()
        createARGame()
    }
    
    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated)
        if GameManager.instance.gameStatus == .fail {
            onFailToCreateNewGame()
        }
    }
    
    override func viewWillAppear(_ animated: Bool) {
        NotificationCenter.default.addObserver(self, selector: #selector(createARGame), name: .newGameCreated, object: nil)
        NotificationCenter.default.addObserver(self, selector: #selector(onGameOver), name: .gameOver, object: nil)
        NotificationCenter.default.addObserver(self, selector: #selector(onFailToCreateNewGame), name: .failToCreateNewGame, object: nil)
        super.viewWillAppear(animated)
        
        // Create a session configuration
        let configuration = ARWorldTrackingConfiguration()
        
        // Run the view's session
        sceneView.session.run(configuration)
        
    }
    
    override func viewWillDisappear(_ animated: Bool) {
        super.viewWillDisappear(animated)
        
        // Pause the view's session
        sceneView.session.pause()
    }
    
    override func viewDidDisappear(_ animated: Bool) {
        super.viewDidDisappear(animated)
        NotificationCenter.default.post(name: .gameDismissed, object: nil)
    }
    
    @objc func createARGame() {
        if let words = GameManager.instance.words {
            let firstWords = words.safePrefix(MAX_WORDS_PER_ROUND)
            updateLearnWordsCounterLabel()
            initializeGame(with: firstWords)
        }
    }
    @objc func onFailToCreateNewGame() {
        performSegue(withIdentifier: "GameStatusDialogSegue", sender: nil)
    }
    
    func nextRoundGame() {
        if let nextWords = GameManager.instance.words?.safeFromIndex(MAX_WORDS_PER_ROUND, startAt: (MAX_WORDS_PER_ROUND * currentRound)) {
            initializeGame(with: nextWords)
        }
        currentRound += 1
    }
    
    @objc func onGameOver() {
        performUIUpdatesOnMain { [weak self] in
            self?.performSegue(withIdentifier: "GameStatusDialogSegue", sender: nil)
        }
    }
    
    func updateLearnWordsCounterLabel() {
        learntWordsCounterLabel.text = learntWordsCounterText
    }
    
    @IBAction func shootButtonWasPressed(_ sender: Any) {
        guard let currentFrame = self.sceneView.session.currentFrame else {
            return
        }
        var translation = matrix_identity_float4x4
        translation.columns.3.z = -0.3
        let bullet = SCNSphere(radius: 0.05)
        let material = SCNMaterial()
        material.diffuse.contents = UIColor.yellow
        bullet.materials = [material]
        let bulletNode = SCNNode(geometry: bullet)
        bulletNode.name = "Bullet"
        bulletNode.physicsBody = SCNPhysicsBody(type: .dynamic, shape: nil)
        bulletNode.physicsBody?.categoryBitMask = BodyType.bullet.rawValue
        bulletNode.physicsBody?.contactTestBitMask = BodyType.word.rawValue
        bulletNode.physicsBody?.isAffectedByGravity = false
        
        bulletNode.simdTransform = matrix_multiply(currentFrame.camera.transform, translation)
        
        let impulseMultiplier:Float = 6
        let forceVector = SCNVector3(bulletNode.worldFront.x * impulseMultiplier,bulletNode.worldFront.y * (impulseMultiplier * 1),bulletNode.worldFront.z * impulseMultiplier)
        
        bulletNode.physicsBody?.applyForce(forceVector, asImpulse: true)
        self.sceneView.scene.rootNode.addChildNode(bulletNode)
        
        // Remove bullets after one second
        DispatchQueue.main.asyncAfter(deadline: .now() + 1) {
            bulletNode.removeFromParentNode()
        }
    }
    @IBAction func exitButtonWasPressed(_ sender: Any) {
        GameManager.instance.updateGameStatus(to: .dismiss)
        performSegue(withIdentifier: "GameStatusDialogSegue", sender: nil)
    }
}

// ARKit extension
extension ARGameViewController: ARSCNViewDelegate {
    
    func sessionWasInterrupted(_ session: ARSession) {
        print("======= Session was interrupted")
    }
    
    func sessionInterruptionEnded(_ session: ARSession) {
        print("======= Session did finish interrupted")
    }
    
    func session(_ session: ARSession, didFailWithError error: Error) {
        print("======= Session did fail: \(error)")
    }
    
    private func createSpinAnimation(duration: Double) -> CABasicAnimation {
        let spin = CABasicAnimation(keyPath: "rotation")
        spin.fromValue = NSValue(scnVector4: SCNVector4(x: 0, y: 1, z: 0, w: 0))
        spin.toValue = NSValue(scnVector4: SCNVector4(x: 0, y: 1, z: 0, w: Float(2 * Double.pi)))
        spin.duration = duration
        spin.repeatCount = .infinity
        return spin
    }
    
    fileprivate func initializeGame(with words:[Word]) {
        var initialWordPosition = SCNVector3(x:0.2, y: -0.5, z: -1)
        for word in words {
            var newWordPosition:SCNVector3 {
                var randomNumber = Float().random(min: -1.3, max: 1.3)
                let oldImagePosition = initialWordPosition
                initialWordPosition = SCNVector3(x: oldImagePosition.x + randomNumber, y: oldImagePosition.y + (randomNumber / 2), z: oldImagePosition.z + randomNumber)
                while(oldImagePosition.distance(vector: initialWordPosition) < 1.6) {
                    randomNumber = Float().random(min: -2.2, max: 2.3)
                    initialWordPosition = SCNVector3(x: oldImagePosition.x + randomNumber, y: oldImagePosition.y + (randomNumber / 2), z: oldImagePosition.z + randomNumber)
                }
                return initialWordPosition
            }
            renderWord(word: word, at: newWordPosition)
        }
    }
    
    private func createWordSceneNode(image:UIImage, word:String) -> SCNNode? {
        if let annotationScn = SCNScene(named: "annotation.scn") {
            if let annotationNode = annotationScn.rootNode.childNode(withName: "parent", recursively: true) {
                if let box = annotationScn.rootNode.childNode(withName: "box", recursively: true) {
                    
                    if let geometry = box.geometry as? SCNBox {
                        geometry.chamferRadius = 0.05
                        // Materials configuration
                        let whiteMaterial = SCNMaterial()
                        whiteMaterial.diffuse.contents = UIColor.white
                        let imageMaterial = SCNMaterial()
                        imageMaterial.diffuse.contents = image
                        imageMaterial.specular.contents = UIColor(white: 255, alpha: 0.6)
                        geometry.materials = [imageMaterial, imageMaterial, imageMaterial, imageMaterial, whiteMaterial, whiteMaterial]
                        box.addAnimation(createSpinAnimation(duration: 45), forKey: "box rotation")
                        
                    }
                }
                if let marker = annotationNode.childNode(withName: "annotation", recursively: true) {
                    marker.addAnimation(createSpinAnimation(duration: 30), forKey: "marker rotation")
                }
                if let textNode = annotationScn.rootNode.childNode(withName: "text", recursively: true) {
                    if let textGeo = textNode.geometry as? SCNText {
                        textGeo.string = word
                    }
//                    textNode.eulerAngles = SCNVector3Make(0, .pi, 0)
                }
                
                annotationNode.physicsBody = SCNPhysicsBody(type: .dynamic, shape: nil)
                annotationNode.physicsBody?.isAffectedByGravity = false
                annotationNode.physicsBody?.mass = 15
                annotationNode.physicsBody?.categoryBitMask = BodyType.word.rawValue
                annotationNode.physicsBody?.contactTestBitMask = BodyType.bullet.rawValue
                return annotationNode
            }
        }
        return nil
    }
    
    func renderWord(word: Word, at position:SCNVector3) {
        
        googleStorage.getImage(imageName: word.image) {[weak self] (image) in
            if let image = image {
                let wordNode = self?.createWordSceneNode(image: image, word: word.name)
                if let wordNode = wordNode {
                    wordNode.position = position
                    wordNode.name = word.id
                    self?.sceneView.scene.rootNode.addChildNode(wordNode)
                }
            }

        }
    }
}

// MARK: Collisions extension
extension ARGameViewController: SCNPhysicsContactDelegate {
    func physicsWorld(_ world: SCNPhysicsWorld, didBegin contact: SCNPhysicsContact) {
        if contact.nodeA.name == "Bullet" {
            GameManager.instance.newWordLearnt(wordId: contact.nodeB.name!)
            if GameManager.instance.learntWords.count == (MAX_WORDS_PER_ROUND * currentRound) {
                nextRoundGame()
            }
            
            contact.nodeB.physicsBody?.isAffectedByGravity = true
            contact.nodeA.physicsBody?.isAffectedByGravity = true
            DispatchQueue.main.asyncAfter(deadline: .now() + 0.25) {
                self.updateLearnWordsCounterLabel()
                contact.nodeB.removeFromParentNode()
                contact.nodeA.removeFromParentNode()
            }
            return
        } else if contact.nodeB.name == "Bullet" {
            GameManager.instance.newWordLearnt(wordId: contact.nodeA.name!)
            if GameManager.instance.learntWords.count == (MAX_WORDS_PER_ROUND * currentRound) {
                nextRoundGame()
            }
            
            contact.nodeA.physicsBody?.isAffectedByGravity = true
            contact.nodeB.physicsBody?.isAffectedByGravity = true
            DispatchQueue.main.asyncAfter(deadline: .now() + 0.25) {
                self.updateLearnWordsCounterLabel()
                contact.nodeB.removeFromParentNode()
                contact.nodeA.removeFromParentNode()
            }
            return
        }
    }
    
}


