//
//  MapViewController.swift
//
//  Created by Laura Sempere on 18/11/2017.
//  Copyright © 2017 laurasempere. All rights reserved.
//

import UIKit
import MapKit

class MapViewController: UIViewController, CLLocationManagerDelegate, Alertable {
    
    @IBOutlet weak var mapView: MKMapView!
    @IBOutlet weak var userProfileButton: UserInitials!
    let MINIMUM_DISTANCE = 50
    
    var locationManager = CLLocationManager()
    var locationUpdateCount = 0
    var places:[GooglePlaces]?
    var selectedPlaces:[GooglePlaces]? {
        get {
            var randomPlaces:[GooglePlaces] = []
            if let places = places {
                let randomPlacesCount = arc4random_uniform(UInt32(places.count))
                var placesIndexes:[Int] {
                    var indexes:[Int] = []
                    for idx in 0..<places.count {
                        indexes.append(idx)
                    }
                    return indexes
                }
                var randomIndexes:Set<Int> = []
                for _ in 0..<randomPlacesCount {
                    randomIndexes.insert(placesIndexes.randomElement)
                }
                for idx in randomIndexes {
                    randomPlaces.append(places[idx])
                }
                return randomPlaces
            }
            return nil
        }
    }
    
    // MARK: Class lifecycle methods
    override func viewDidLoad() {
        super.viewDidLoad()
        mapView.delegate = self
        getPlacesToPlay()
        userProfileButton.onUserProfile = {[weak self] in
            self?.performSegue(withIdentifier: "UserProfileSegue", sender: nil)
        }
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(true)
        self.navigationController?.navigationBar.isHidden = true
        
        // Check location authorisation status
        if AuthService.instance.getCurrentUser() != nil {
            checkLocationAuthorisationStatus()
        }
    }
    
    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated)
        userProfileButton.setUserImage()
    }
        
    // MARK: LocationManager delegate
    
    func locationManager(_ manager: CLLocationManager, didChangeAuthorization status: CLAuthorizationStatus) {
        checkLocationAuthorisationStatus()
        getPlacesToPlay()
    }

    func locationManager(_ manager: CLLocationManager, didUpdateLocations locations: [CLLocation]) {
        if locationUpdateCount < 2 {
            setCenter(for: manager)
            locationUpdateCount += 1
        } else {
            manager.stopUpdatingLocation()
        }
    }
    
    // MARK: Helper methods
    func getPlacesToPlay() {
        if let longitude = locationManager.location?.coordinate.longitude, let latitude = locationManager.location?.coordinate.latitude {
            GooglePlacesService.instance.getPlacesFor(longitude: Double(longitude), latitude: Double(latitude), completion: {[weak self](places, error) in
                if let places = places, !places.isEmpty {
                    self?.places = places
                    self?.addAnnotations(for: places)
                } else {
                    self?.showAlert("No places ARound", "Sorry there are no words ARound your current location")
                }
            })
        }
    }
    
    func checkLocationAuthorisationStatus(){
        locationManager.delegate = self
        if CLLocationManager.authorizationStatus() == .authorizedWhenInUse {
            locationManager.desiredAccuracy = kCLLocationAccuracyBest
            mapView.showsUserLocation = true
            mapView.userTrackingMode = .follow
            locationManager.startUpdatingLocation()
            
        } else {
            // Request user authorization to get his/her location
            locationManager.requestWhenInUseAuthorization()
            
        }
    }
    
    func setCenter(for locationManger: CLLocationManager) {
        if let coordinate = locationManger.location?.coordinate {
            let region = MKCoordinateRegionMakeWithDistance(coordinate, 500, 500)
            mapView.setRegion(region, animated: true)
        }
    }
    
    func addAnnotations(for places:[GooglePlaces]) {
        places.map { place in
            if let placeTypes = place.types {
                guard let primaryPlaceType = getPrimaryPlaceType(placeTypes: placeTypes) else { return }
                guard let lat = place.geometry?.location?.lat, let lon = place.geometry?.location?.lng else { return }
                let coord = CLLocationCoordinate2D(latitude: CLLocationDegrees(lat), longitude: CLLocationDegrees(lon))
                let annotation = PlaceTypeAnnotation(coord: coord, placeType: primaryPlaceType)
            
                mapView.addAnnotation(annotation)
            }
        }
    }
        
    func searchPlaces() {
        let localSearchRequest = MKLocalSearchRequest()
        localSearchRequest.naturalLanguageQuery = ""
        let localSearch = MKLocalSearch(request: localSearchRequest)
        localSearch.start { (response, error) in
            if error != nil {
                
            } else {
                for item in response!.mapItems {
                    print(item.name!)
                    
                }
            }
        }
    }
        
    @IBAction func centerLocationButtonWasPressed(_ sender: Any) {
        setCenter(for: locationManager)
    }
    
}
// MARK: MKMapView delegate methods
extension MapViewController: MKMapViewDelegate {
    
    func mapView(_ mapView: MKMapView, viewFor annotation: MKAnnotation) -> MKAnnotationView? {
        if annotation is MKUserLocation {
            return nil
        }
        guard let locationTypeAnnotation  = annotation as? PlaceTypeAnnotation else { return nil }
        guard let locationImage = locationTypeAnnotation.placeType.getImage() else { return nil }
        let annotationView = MKAnnotationView(annotation: annotation, reuseIdentifier: nil)
        annotationView.image =  UIImage(named: locationImage)
        var frame = annotationView.frame
        frame.size.height = 40
        frame.size.width = 40
        annotationView.frame = frame
        return annotationView
    }
        
    func mapView(_ mapView: MKMapView, didSelect view: MKAnnotationView) {
        mapView.deselectAnnotation(view.annotation!, animated: true)
        if view.annotation! is MKUserLocation {
            return
        }
        guard let placeTypeAnnotation = view.annotation as? PlaceTypeAnnotation else { return }
        let placeLocation = CLLocation(latitude: view.annotation!.coordinate.latitude, longitude: view.annotation!.coordinate.longitude)
        let distance = Int((locationManager.location?.distance(from: placeLocation))!)
        
        if distance < MINIMUM_DISTANCE {
            GameManager.instance.createNewGame(for: "A1", at: placeTypeAnnotation.placeType)
            GameManager.instance.updateGameStatus(to: .start)
            
            if let annotation = view.annotation {
                mapView.removeAnnotation(annotation)
            }
            if mapView.annotations.count < 5 {
                getPlacesToPlay()
            }
            performSegue(withIdentifier: "StartGameConfirmationSegue", sender: placeTypeAnnotation)
        } else {
            GameManager.instance.placeType = placeTypeAnnotation.placeType
            GameManager.instance.updateGameStatus(to: .farAway)
            performSegue(withIdentifier: "StartGameConfirmationSegue", sender: placeTypeAnnotation)
        }
    }
}

 

