//
//  AuthService.swift
//
//  Created by Laura Sempere on 18/11/2017.
//  Copyright © 2017 laurasempere. All rights reserved.
//

import Foundation
import FBSDKCoreKit
import FBSDKLoginKit
import Firebase
import GoogleSignIn

protocol GoogleLoginComplete {
    func googleLoginComplete(loginComplete: Bool)
}

class AuthService: NSObject, GIDSignInDelegate, GIDSignInUIDelegate {
    
    // Singleton
    static let instance = AuthService()
    
    // Protocol to communicate google login completion
    var googleLoginCompleteDelegate: GoogleLoginComplete?
    
    func loginWith(credential:AuthCredential, completion: @escaping(_ status: Bool, _ error: Error?) -> ()) {
        Auth.auth().signIn(with: credential) { (user, error) in
            guard let user = user else {
                completion(false, error)
                return
            }
            
            var userData = ["provider": credential.provider, "email": user.email, "displayName": user.displayName]
            if let photoUrl = user.providerData.first?.photoURL?.absoluteString {
                userData["photo"] = photoUrl
            }
            DataService.instance.createDBUser(uid: user.uid, userData: userData)
            completion(true, nil)
        }
    }
    
    // MARK: Facebook login
    func facebookLogin(sender:UIViewController, completion: @escaping(_ status: Bool, _ error: Error?) -> ()) {
        let loginManager = FBSDKLoginManager()
        
        loginManager.logIn(withReadPermissions: ["public_profile","email"], from: sender) {[weak self](result, loginError) in
            
            if let err = loginError {
                completion(false, err)
            } else {
                let credential = FacebookAuthProvider.credential(withAccessToken: FBSDKAccessToken.current().tokenString)
                self?.loginWith(credential: credential, completion: completion)
                
            }
        }
    }
    
    func googleLogin() {
        GIDSignIn.sharedInstance().uiDelegate = self
        GIDSignIn.sharedInstance().signIn()
    }
    
    // MARK: Google Signin UI delegate methods
    func sign(_ signIn: GIDSignIn!, didSignInFor user: GIDGoogleUser!, withError error: Error!) {
        guard let user:GIDGoogleUser = user else { return }
        guard let authentication = user.authentication else { return }
        let credential = GoogleAuthProvider.credential(withIDToken: authentication.idToken,
                                                       accessToken: authentication.accessToken)
        loginWith(credential: credential) {[weak self] (status, error) in
            self?.googleLoginCompleteDelegate?.googleLoginComplete(loginComplete: status)
        }
    }
    
    func sign(inWillDispatch signIn: GIDSignIn!, error: Error!) {
        self.googleLoginCompleteDelegate?.googleLoginComplete(loginComplete: false)
    }
    
    func sign(_ signIn: GIDSignIn!, present viewController: UIViewController!) { }
    
    func sign(_ signIn: GIDSignIn!, dismiss viewController: UIViewController!) { }
    
    func logoutUser(logoutComplete: (_ status: Bool) -> ()) {
        do {
           try Auth.auth().signOut()
            logoutComplete(true)
            DataService.instance.resetUserDetails()
            NotificationCenter.default.post(name: .logout, object: true)
        } catch {
            logoutComplete(false)
            print(error)
        }
        
    }
    
    func getCurrentUser() -> User? {
        return Auth.auth().currentUser
    }
    
}
