//
//  DataService.swift
//
//  Created by Laura Sempere on 18/11/2017.
//  Copyright © 2017 laurasempere. All rights reserved.
//

import Foundation
import Firebase

let DB_BASE = Database.database().reference()
let POINTS_FOR_WORD = 10

struct GetGameResponse: Codable {
    let words:[String:Word]
}

class DataService {
    
    // singleton class
    static let instance = DataService()
    let decoder = JSONDecoder()
    
    private(set) var userDetails:UserDetails?
    
    // Firebase Database references
    private let REF_BASE = DB_BASE
    private let REF_USERS = DB_BASE.child("users")
    private let REF_LOCATION = DB_BASE.child("location")
    private let REF_LEVELS = DB_BASE.child("levels")
    private let REF_WORDS = DB_BASE.child("words")
    private let REF_GAME = DB_BASE.child("game")
    
    func resetUserDetails () {
        userDetails = nil
    }
    
    func createDBUser(uid:String, userData: Dictionary<String, Any>) {
        REF_USERS.observeSingleEvent(of: .value) { (snapshot) in
            if snapshot.hasChild(uid) {
            self.REF_USERS.child(uid).updateChildValues(userData)
            } else {
                var newUserData = userData
                newUserData["points"] = 0
                newUserData["level"] = "A1"
                self.REF_USERS.child(uid).updateChildValues(newUserData)
                self.getUserDetails(completion: nil)
            }
        }
        
    }
    
    func addLearntWord(wordId: String) {
        guard let userId = Auth.auth().currentUser?.uid else { return }
        let refCurrentUser = REF_USERS.child("\(userId)")
        refCurrentUser.child("learntWords").updateChildValues([wordId : true])
        refCurrentUser.observeSingleEvent(of: .value) { (snapshot) in
            let points = snapshot.childSnapshot(forPath: "points").value as? Int ?? 0
            let updatedPoints = points + POINTS_FOR_WORD
            refCurrentUser.updateChildValues(["points": updatedPoints])
            if (updatedPoints / 100) > (points / 100) {
                self.updateExtraContent(available: true)
            } else {
                guard self.userDetails?.extraContentAvailable == nil else { return }
                self.updateExtraContent(available: false)
                
//                if let extraContentAvailable = self.userDetails?.extraContentAvailable  {
//
//                } else {
//                    self.updateExtraContent(available: false)
//                }
                
                
            }
          
        }
    }
    
    func updateExtraContent(available: Bool) {
        guard let userId = Auth.auth().currentUser?.uid else { return }
        let refCurrentUser = REF_USERS.child("\(userId)")
        refCurrentUser.updateChildValues(["extraContentAvailable" : available])
    }
    
    func getGame(for levelId: String, at locationId: String, completion: @escaping (_ words: [Word]?, _ error: String?) -> Void) {
        REF_GAME.child("\(levelId)/\(locationId)/").observeSingleEvent(of: .value) { (snapshot) in
            guard let data = self.getDataFromDataSnapshot(snapshot: snapshot) else {
                completion(nil, "No words available for current location")
                return
            }
            do {
                let resp = try self.decoder.decode(GetGameResponse.self, from: data)
                let words = resp.words.map{$0.value}
                completion(words, nil)
            } catch let err {
                completion(nil, err.localizedDescription)
            }
        }
    }
    
    func getUserDetails(completion: (() -> Void)?) {
        if let userUID = AuthService.instance.getCurrentUser()?.uid {
            REF_USERS.child(userUID).observeSingleEvent(of: .value, with: { (snapshot) in
                guard let data = self.getDataFromDataSnapshot(snapshot: snapshot) else { return }
                do {
                    self.userDetails = try self.decoder.decode(UserDetails.self, from: data)
                } catch {
                    print("ERROR decoding user details:\(error)")
                }
                completion?()
            })
        }
    }
    
    func getLearntWords(completion: @escaping(_ words: [Word]?, _ error: String?) -> Void) {
        guard let learntWords = userDetails?.learntWords else { return }
        let learntWordsIds = learntWords.map{$0.0}
    
        REF_WORDS.observeSingleEvent(of: .value) { (snapshot) in
            guard let wordsSnapshot = snapshot.children.allObjects as? [DataSnapshot] else { return }
            var words:[Word] = []
                for word in wordsSnapshot {
                    if learntWordsIds.contains(word.key) {
                        guard let data = self.getDataFromDataSnapshot(snapshot: word) else { return }
                        let decodedWord = try? self.decoder.decode(Word.self, from: data)
                        if let decoded = decodedWord {
                            words.append(decoded)
                        }
                    }
                }
            completion(words, nil)
        }
    }
    
}

// Helper extension
extension DataService {
    func getDataFromDataSnapshot(snapshot: DataSnapshot) -> Data? {
        guard let object = snapshot.value as? [String: Any] else {
            return nil
        }
        return try? JSONSerialization.data(withJSONObject: object, options: .prettyPrinted)
    }
}
